package com.contentsquare.plugins.cordova;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.webkit.WebView;

import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaWebView;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.contentsquare.android.Contentsquare;
import com.contentsquare.android.api.model.Transaction;
import com.contentsquare.android.api.CsWebViewManager;

/**
 * This class echoes a string called from JavaScript.
 */
public class CDVContentsquarePlugin extends CordovaPlugin {

    private class WebViewInjector implements Runnable {
        private CordovaWebView webView;

        @Override
        public void run() {
            try {
                WebView realWebView = (WebView) this.webView.getEngine().getView();
                CsWebViewManager.injectEventTrackingInterface(realWebView);
                Log.d("CSLIB", "WebView Injected !!");
            } catch (Exception e) {
                Log.e("CSLIB", e.getMessage());
            }
        }

        public void setWebView(CordovaWebView webView) {
            this.webView = webView;
        }
    }

    @Override
    public void pluginInitialize() {
        WebViewInjector webViewInjector = new WebViewInjector();
        Handler webViewInjectorHandler = new Handler(Looper.getMainLooper());
        webViewInjector.setWebView(this.webView);
        webViewInjectorHandler.post(webViewInjector);
    }

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        if (action.equals("optIn")) {
            this.optIn(callbackContext);
            return true;
        } else if (action.equals("optOut")) {
            this.optOut(callbackContext);
            return true;
        } else if (action.equals("sendScreenName")) {
            String name = args.getString(0);
            this.sendScreenName(name, callbackContext);
            return true;
        } else if (action.equals("sendTransaction")) {
            String transactionValue = args.getString(0);
            String transactionCurrency = args.getString(1);
            String transactionId = args.getString(2);
            this.sendTransaction(transactionValue, transactionCurrency, transactionId, callbackContext);
            return true;
        } else if (action.equals("sendDynamicVarWithStringValue")) {
            String dynVarKey = args.getString(0);
            String dynVarValue = args.getString(1);
            this.sendDynamicVarWithStringValue(dynVarKey, dynVarValue, callbackContext);
            return true;
        } else if (action.equals("sendDynamicVarWithIntValue")) {
            String dynVarKey = args.getString(0);
            Integer dynVarValue = args.getInt(1);
            this.sendDynamicVarWithIntValue(dynVarKey, dynVarValue, callbackContext);
            return true;
        }
        return false;
    }

    private void optIn(CallbackContext callbackContext) {
        Context context = this.cordova.getActivity().getApplicationContext();
        Contentsquare.optIn(context);
        callbackContext.success("CDVContentsquarePlugin processing 'optIn'");
    }

    private void optOut(CallbackContext callbackContext) {
        Context context = this.cordova.getActivity().getApplicationContext();
        Contentsquare.optOut(context);
        callbackContext.success("CDVContentsquarePlugin processing 'optOut'");
    }

    private void sendScreenName(String name, CallbackContext callbackContext) {
        try {
            Contentsquare.send(name);
            callbackContext.success("CDVContentsquarePlugin processing 'sendScreenName' " + name);
        } catch (Exception e) {
            callbackContext.error(e.getMessage());
        }

    }

    private void sendTransaction(String transactionValue, String transactionCurrency, String transactionId,
            CallbackContext callbackContext) {
        try {
            if (transactionId == null || transactionId.isEmpty()) {
                Contentsquare
                        .send(Transaction.builder(Float.parseFloat(transactionValue), transactionCurrency).build());
            } else {
                Contentsquare.send(Transaction.builder(Float.parseFloat(transactionValue), transactionCurrency)
                        .id(transactionId).build());
            }
            callbackContext.success("CDVContentsquarePlugin processing 'sendTransaction' " + transactionValue + " "
                    + transactionCurrency);
        } catch (Exception e) {
            callbackContext.error(e.getMessage());
        }
    }

    private void sendDynamicVarWithStringValue(String dynVarKey, String dynVarValue, CallbackContext callbackContext) {
        try {
            Contentsquare.send(dynVarKey, dynVarValue);
            callbackContext.success("CDVContentsquarePlugin processing 'sendDynamicVarWithStringValue' " + dynVarKey
                    + ": " + dynVarValue);
        } catch (Exception e) {
            callbackContext.error(e.getMessage());
        }
    }

    private void sendDynamicVarWithIntValue(String dynVarKey, Integer dynVarValue, CallbackContext callbackContext) {
        try {
            Contentsquare.send(dynVarKey, dynVarValue);
            callbackContext.success(
                    "CDVContentsquarePlugin processing 'sendDynamicVarWithIntValue' " + dynVarKey + ": " + dynVarValue);
        } catch (Exception e) {
            callbackContext.error(e.getMessage());
        }
    }

    private void handleURL(String url, CallbackContext callbackContext) {
        callbackContext.success("CDVContentsquarePlugin processing 'handleURL'");
    }
}
