/*
* Notes: The @objc shows that this class & function should be exposed to Cordova.
*/
import ContentsquareModule
import WebKit

@objc(CDVContentsquarePlugin) class CDVContentsquarePlugin : CDVPlugin {

   // Inject WkWebView
  open override func pluginInitialize() {
      if let realWebView = self.webView as? WKWebView {
            Contentsquare.register(webView: realWebView)
            print("Webview injection successful")
      } else {
            print("Webview injection failed")
      }
  }

  @objc(optIn:) // Declare your function name.
  func optIn(command: CDVInvokedUrlCommand) { // write the function code.
    /* 
     * Always assume that the plugin will fail.
     * Even if in this example, it can't.
     */
    // Set the plugin result to fail.
    var pluginResult = CDVPluginResult (status: CDVCommandStatus_ERROR, messageAs: "CDVContentsquarePlugin 'optIn' method Failed");
    Contentsquare.optIn();
    // Set the plugin result to succeed.
    pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "CDVContentsquarePlugin processing 'optIn'");
    // Send the function result back to Cordova.
    self.commandDelegate!.send(pluginResult, callbackId: command.callbackId);
  }

  @objc(optOut:) // Declare your function name.
  func optOut(command: CDVInvokedUrlCommand) { // write the function code.
    var pluginResult = CDVPluginResult (status: CDVCommandStatus_ERROR, messageAs: "CDVContentsquarePlugin 'optOut' method Failed");
    Contentsquare.optOut();
    pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "CDVContentsquarePlugin processing 'optOut'");
    self.commandDelegate!.send(pluginResult, callbackId: command.callbackId);
  }


  @objc(sendScreenName:)
  func sendScreenName(command: CDVInvokedUrlCommand) {
    var pluginResult = CDVPluginResult (status: CDVCommandStatus_ERROR, messageAs: "CDVContentsquarePlugin 'sendScreenName' method Failed");
    let name = command.arguments[0] as? String ?? ""
    Contentsquare.send(screenViewWithName: name);
    pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "CDVContentsquarePlugin processing 'sendScreenName'");
    self.commandDelegate!.send(pluginResult, callbackId: command.callbackId);
  }

  @objc(handleURL:)
  func handleURL(command: CDVInvokedUrlCommand) {
    var pluginResult = CDVPluginResult (status: CDVCommandStatus_ERROR, messageAs: "CDVContentsquarePlugin 'handleURL' method Failed");
    let urlSt = command.arguments[0] as? String ?? ""
    let url = URL(string: urlSt);
    Contentsquare.handle(url: url!);
    pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "CDVContentsquarePlugin processing 'handleURL'");
    self.commandDelegate!.send(pluginResult, callbackId: command.callbackId);
  }

  @objc(sendTransaction:)
  func sendTransaction(command: CDVInvokedUrlCommand) {
    var pluginResult = CDVPluginResult (status: CDVCommandStatus_ERROR, messageAs: "CDVContentsquarePlugin 'sendTransaction' method Failed");
    let transactionValue = command.arguments[0] as? Float ?? 0
    let transactionCurrency = command.arguments[1] as? String ?? ""
    let transactionId = command.arguments[2] as? String ?? ""
    
    Contentsquare.send(transaction: CustomerTransaction(id: transactionId, value: transactionValue, currency: transactionCurrency))
  
    pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "CDVContentsquarePlugin processing 'sendTransaction' with currency: "+transactionCurrency);
    self.commandDelegate!.send(pluginResult, callbackId: command.callbackId);
  }

  @objc(sendDynamicVarWithStringValue:)
  func sendDynamicVarWithStringValue(command: CDVInvokedUrlCommand) {
    var pluginResult = CDVPluginResult (status: CDVCommandStatus_ERROR, messageAs: "CDVContentsquarePlugin 'sendDynamicVarWithStringValue' method Failed");
    let key = command.arguments[0] as? String ?? ""
    let value = command.arguments[1] as? String ?? ""
   
    Contentsquare.send(dynamicVar: DynamicVar(key: key, value: value))
    pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "CDVContentsquarePlugin processing 'sendDynamicVarWithStringValue'");
    self.commandDelegate!.send(pluginResult, callbackId: command.callbackId);
  }

  @objc(sendDynamicVarWithIntValue:)
  func sendDynamicVarWithIntValue(command: CDVInvokedUrlCommand) {
    var pluginResult = CDVPluginResult (status: CDVCommandStatus_ERROR, messageAs: "CDVContentsquarePlugin 'sendDynamicVarWithIntValue' method Failed");
    let key = command.arguments[0] as? String ?? ""
    let value = command.arguments[1] as? UInt32 ?? 0
   
    Contentsquare.send(dynamicVar: DynamicVar(key: key, value: value))
    pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "CDVContentsquarePlugin processing 'sendDynamicVarWithIntValue'");
    self.commandDelegate!.send(pluginResult, callbackId: command.callbackId);
  }


        
}